# -*- coding: UTF-8 -*-

import sys
from collections import deque
import time

"""
call interface "/proc/stat"

|cpu	|user	|nice	|system	|idle		|iowait	|irq	|softirq|steal	|guest	|guest_nice	|

|cpu	|742944	|162	|164377	|967497954	|433345	|0	|3271	|159478	|0	|0         	|
	
|cpu0	|..	|..	|..	|..		|..	|..	|..	|..	|..	|..		|

cpu idle usage = idle / (user + nice + system + idle + iowait + irq + softirq + steal + guest + guest_nice)
cpu usage      = ( 1 -  cpu idle usage ) * 100

"""

def compute(core=None):
    with open('/proc/stat', 'r') as f:
        exitflag = False
        cpuInfo={}
        for line in f:
            value = line.rstrip('\n').split()
            if value[0] != core:
                continue
            else:
                value_list = deque(value)
                value_list.popleft()
                cpuInfo['user'] = float(value_list[0])
                cpuInfo['nice'] = float(value_list[1])
                cpuInfo['system'] = float(value_list[2])
                cpuInfo['idle'] = float(value_list[3])
                cpuInfo['iowait'] = float(value_list[4])
                cpuInfo['irq'] = float(value_list[5])
                cpuInfo['softirq'] = float(value_list[6])
                cpuInfo['total'] = sum([int(x) for x in value_list])
                cpuInfo['usage'] = cpuInfo['total'] - cpuInfo['idle']
                exitflag = True
    if exitflag:
        return cpuInfo
    else:
        print("core[%s] not exit!"%core)
        sys.exit()

def cpu_info(core=None,opt=None):
    core_old = compute(core)
    time.sleep(2)
    core_new = compute(core)
    opt_add =(core_new[opt]-core_old[opt])
    total_add = (core_new['total']-core_old['total'])
    #print used, total
    if total_add == 0:
        percent = 0
    else:
        percent =  (opt_add/total_add) * 100

    return round(percent, 2)

def main(core=None, opt=None):
    print(cpu_info(core, opt))


if __name__ == '__main__':
    opts = ['usage', 'idle']
    if len(sys.argv) != 3 or sys.argv[2] not in opts:
        print("arg err!(cpu_zabbix cpu|cpu0|cpu1... idle|usage)")
        sys.exit()
    
    main(core=sys.argv[1], opt=sys.argv[2])




