# -*- coding: UTF-8 -*-

import time
import re
import os
from functools import reduce

SLEEPTIME = 2

def tonum(n):
    if n.isdigit():
        return int(n)
    return n

def get_diskusage():
    lines = open("/proc/partitions").readlines()[2:]
    partitions = set([line.split()[-1] for line in lines if not line.strip()[-1].isdigit()])
    def line_to_dict(line):
        major, minor, dev, r_ios, r_merges, r_sec, r_ticks, w_ios, w_merges, w_sec, w_ticks, ios_pgr, tot_ticks, rq_ticks = line.split()[:14]
        del line
        d={}
        d['major'] = major
        d['minor'] = minor
        d['dev'] = dev
        d['r_ios'] = float(r_ios)
        d['r_merges'] = float(r_merges)
        d['r_sec'] = float(r_sec)
        d['r_ticks'] = float(r_ticks)
        d['w_ios'] = float(w_ios)
        d['w_merges'] = float(w_merges)
        d['w_sec'] = float(w_sec)
        d['w_ticks'] = float(w_ticks)
        d['ios_pgr'] = float(ios_pgr)
        d['tot_ticks'] = float(tot_ticks)
        d['rq_ticks'] = float(rq_ticks)
        d['ts'] = time.time()
        return d

    lines = open("/proc/diskstats").readlines()
    stats = [line_to_dict(line) for line in lines]
    new_stats={}
    for stat in stats:
        if stat['dev'] in partitions:
            new_stats[stat['dev']]=stat
    return new_stats

def compute(last, curr, key):
    SECTOR_SIZE = 512
    stat = {}

    def diff(field):
        return (curr[field] - last[field]) / (curr["ts"] - last["ts"])

    stat['rrqm/s']   = round(diff('r_merges'), 2)
    stat['wrqm/s']   = round(diff('w_merges'), 2)
    stat['r/s']      = round(diff('r_ios'), 2)
    stat['w/s']      = round(diff('w_ios'), 2)
    stat['rkB/s']    = round(diff('r_sec') * SECTOR_SIZE / 1024, 2)
    stat['wkB/s']    = round(diff('w_sec') * SECTOR_SIZE / 1024, 2)

    stat['avqqu-sz'] = round(diff('rq_ticks') / 1000, 2)
    #print 'tot_ticks', curr['tot_ticks'], last['tot_ticks']
    stat['util']     = round(diff('tot_ticks')/10, 2) #???

    if diff('r_ios') + diff('w_ios') > 0:
        stat['avgrq-sz'] = round(( diff('r_sec') + diff('w_sec') ) / ( diff('r_ios') + diff('w_ios') ), 2)
        stat['await']    = round(( diff('r_ticks') + diff('w_ticks') ) / ( diff('r_ios') + diff('w_ios') ), 2)
        stat['svctm']    = round(diff('tot_ticks') / ( diff('r_ios') + diff('w_ios') ), 2)
    else:
        stat['avgrq-sz'] = 0
        stat['await']    = 0
        stat['svctm']    = 0

    return stat[key]

def serial2name(s):
    lines = open("/proc/partitions").readlines()[2:]
    partitions = set([line.split()[-1] for line in lines if not line.strip()[-1].isdigit()])
    diskuuiddata = []
    for disk in partitions:
        if disk.startswith('sd') or disk.startswith('nvme'):
            diskuuidinfo=os.popen('udevadm info --query=all --name=%s | grep SERIAL_SHORT' %(disk)).readlines()[0].strip('\n')
            if diskuuidinfo[diskuuidinfo.index('=')+1:] == s:
                    return disk
        elif disk.startswith('vd'):
            diskuuidinfo=os.popen('udevadm info --query=all --name=%s | grep SERIAL' %(disk)).readlines()[0].strip('\n')
            if diskuuidinfo[diskuuidinfo.index('=')+1:] == s:
                return disk
    return None

def disk_usage(dname):
    old = get_diskusage()
    time.sleep(SLEEPTIME)
    new = get_diskusage()

    if dname:
        return compute(old[dname], new[dname], 'rkB/s')
    else:
        return reduce(lambda x, y:x+y, [compute(old[dname], new[dname], 'rkB/s') for dname in new])

def getdiskagentid(ppid):
    with open('/proc/%s/status'%ppid, 'r') as ppidstatus:
        for line in ppidstatus:
            if line.startswith('PPid'):
                agentid  = int(line.split()[1])
                break
            else:
                continue
    return agentid

def getdiskuuid(diskagentid):
    with open('/proc/%s/cmdline'%diskagentid, 'r') as ppidcmdline:
        diskprocessconf = ppidcmdline.readline()

    if '#' not in diskprocessconf:
        name = None
    else:
        diskuuid = re.findall(r"#(.+?).conf", diskprocessconf)[0]
        name = serial2name(diskuuid)

    print(disk_usage(name))

def get_os_name():

    with open('/proc/version', 'r') as ver:
        for line in ver:
            if "Red Hat" in line:
                osname = 'redhat'
                diskagentid = getdiskagentid(os.getppid())
                getdiskuuid(diskagentid)
                break
            else:
                osname = 'ubuntu'
                diskagentid = getdiskagentid(getdiskagentid(os.getppid()))
                getdiskuuid(diskagentid)
                break
    
if __name__ =='__main__':

    get_os_name()


