# -*- coding: UTF-8 -*-

import sys
from collections import OrderedDict

"""
call interface "/proc/meminfo"

MemTotal:       32781468 kB
MemFree:        29419780 kB
Buffers:             888 kB
Cached:          2537448 kB

formula:
mem usage	= (MemTotal-(MemFree+Buffers+Cached)) / MemTotal * 100

"""

def compute(memtype=None):
    meminfo = OrderedDict()
    with open('/proc/meminfo', 'r') as f:

        for line in f:
            ll = line.rstrip('\n').split(':')
            meminfo[ll[0]] = ll[1].strip('kB')
    if memtype == 'physical':
        total = int(meminfo['MemTotal'])
        #print float(meminfo['MemFree']), float(meminfo['Buffers']), float(meminfo['Cached'])
        idle = float(meminfo['MemFree']) + float(meminfo['Buffers']) + float(meminfo['Cached'])
    else:
        total = int(meminfo['SwapTotal'])
        idle  = float(meminfo['SwapFree'])

    return total, (total-idle)

def mem_usage():
    total,used = compute('physical')
    if total == 0:
        percent = 0
    else:
        percent = (used/total) * 100

    return round(percent, 2)

def main():
    print(mem_usage())

if __name__ == '__main__':
    main()
